@@ -4,7 +4,7 @@
 #ifndef USE_LIST_SO
 #include "list/list.h"
 #else
-#include <list/list.h>
+#include <clibs/list.h>
 #endif /* USE_LIST_SO */
 
 #endif /* OIDC_LIST_H */
