@@ -8,6 +8,8 @@
 #define TOKENPARSEMODE_SAVE_AT_IF(X) ((X) ? 0x01 : 0)
 #define TOKENPARSEMODE_RETURN_AT 0x02
 #define TOKENPARSEMODE_RETURN_ID 0x04
+#define TOKENPARSEMODE_RETURN_MT 0x08
+#define TOKENPARSEMODE_SAVE_MT 0x08
 
 char* generatePostData(char* k1, char* v1, ...);
 char* generatePostDataFromList(list_t* list);
