@@ -7,8 +7,8 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
-#include <utils/uriUtils.h>
 
+#include "config_updater.h"
 #include "defines/ipc_values.h"
 #include "defines/oidc_values.h"
 #include "defines/settings.h"
@@ -25,11 +25,6 @@
 #include "oidc-agent/oidcp/passwords/password_store.h"
 #include "oidc-agent/oidcp/proxy_handler.h"
 #include "oidc-agent/oidcp/start_oidcd.h"
-#include "utils/oidc/device.h"
-#ifndef __APPLE__
-#include "privileges/agent_privileges.h"
-#endif
-#include "config_updater.h"
 #include "utils/agentLogger.h"
 #include "utils/crypt/crypt.h"
 #include "utils/db/connection_db.h"
@@ -37,9 +32,14 @@
 #include "utils/json.h"
 #include "utils/listUtils.h"
 #include "utils/memory.h"
+#include "utils/oidc/device.h"
 #include "utils/printer.h"
 #include "utils/printerUtils.h"
 #include "utils/string/stringUtils.h"
+#include "utils/uriUtils.h"
+#ifdef __MSYS__
+#include "utils/registryConnector.h"
+#endif
 
 struct connection* unix_listencon;
 
@@ -57,19 +57,19 @@ int main(int argc, char** argv) {
   if (arguments.debug) {
     logger_setloglevel(DEBUG);
   }
-#ifndef __APPLE__
-  if (arguments.seccomp) {
-    initOidcAgentPrivileges(&arguments);
-  }
-#endif
   initCrypt();
   if (arguments.kill_flag) {
-    char* pidstr = getenv(OIDC_PID_ENV_NAME);
+#ifdef __MSYS__
+    char* pidstr = getRegistryValue(OIDC_PID_ENV_NAME);
+#else
+    char* pidstr = oidc_strcopy(getenv(OIDC_PID_ENV_NAME));
+#endif
     if (pidstr == NULL) {
       printError("%s not set, cannot kill Agent\n", OIDC_PID_ENV_NAME);
       exit(EXIT_FAILURE);
     }
     pid_t pid = strToInt(pidstr);
+    secFree(pidstr);
     if (0 == pid) {
       printError("%s not set to a valid pid: %s\n", OIDC_PID_ENV_NAME, pidstr);
       exit(EXIT_FAILURE);
@@ -78,12 +78,23 @@ int main(int argc, char** argv) {
       perror("kill");
       exit(EXIT_FAILURE);
     } else {
+#ifdef __MSYS__
+      char* oidcSockEnvName = getRegistryValue(OIDC_PID_ENV_NAME);
+      unlink(oidcSockEnvName);
+      rmdir(dirname(oidcSockEnvName));
+      secFree(oidcSockEnvName);
+      removeRegistryEntry(OIDC_SOCK_ENV_NAME);
+      removeRegistryEntry(OIDC_PID_ENV_NAME);
+      printStdout("oidc-agent (Process ID %d) killed\n", pid);
+      exit(EXIT_SUCCESS);
+#else
       unlink(getenv(OIDC_SOCK_ENV_NAME));
       rmdir(dirname(getenv(OIDC_SOCK_ENV_NAME)));
       printStdout("unset %s;\n", OIDC_SOCK_ENV_NAME);
       printStdout("unset %s;\n", OIDC_PID_ENV_NAME);
       printStdout("echo Agent pid %d killed;\n", pid);
       exit(EXIT_SUCCESS);
+#endif
     }
   }
   if (arguments.status) {
@@ -114,14 +125,37 @@ int main(int argc, char** argv) {
   if (!arguments.console) {
     pid_t daemon_pid = daemonize();
     if (daemon_pid > 0) {
-      // Export PID of new daemon
+// Export PID of new daemon
+#ifdef __MSYS__
+      char daemon_pid_string[12];
+      sprintf(daemon_pid_string, "%d", daemon_pid);
+      createOrUpdateRegistryEntry(OIDC_PID_ENV_NAME, daemon_pid_string);
+      createOrUpdateRegistryEntry(OIDC_SOCK_ENV_NAME,
+                                  unix_listencon->server->sun_path);
+      printStdout("%s=%s\n", OIDC_SOCK_ENV_NAME,
+                  unix_listencon->server->sun_path);
+      printStdout("%s=%s\n", OIDC_PID_ENV_NAME, daemon_pid_string);
+      exit(EXIT_SUCCESS);
+#else
       printEnvs(unix_listencon->server->sun_path, daemon_pid, arguments.quiet,
                 arguments.json);
       exit(EXIT_SUCCESS);
+#endif
     }
   } else {
+#ifdef __MSYS__
+    char daemon_pid_string[12];
+    sprintf(daemon_pid_string, "%d", getpid());
+    createOrUpdateRegistryEntry(OIDC_PID_ENV_NAME, daemon_pid_string);
+    createOrUpdateRegistryEntry(OIDC_SOCK_ENV_NAME,
+                                unix_listencon->server->sun_path);
+    printStdout("%s=%s\n", OIDC_SOCK_ENV_NAME,
+                unix_listencon->server->sun_path);
+    printStdout("%s=%s\n", OIDC_PID_ENV_NAME, daemon_pid_string);
+#else
     printEnvs(unix_listencon->server->sun_path, getpid(), arguments.quiet,
               arguments.json);
+#endif
   }
 
   agent_state.defaultTimeout = arguments.lifetime;
@@ -137,7 +171,7 @@ int main(int argc, char** argv) {
 _Noreturn void handleClientComm(struct ipcPipe          pipes,
                                 const struct arguments* arguments) {
   connectionDB_new();
-  connectionDB_setFreeFunction((void(*)(void*)) & _secFreeConnection);
+  connectionDB_setFreeFunction((void (*)(void*)) & _secFreeConnection);
   connectionDB_setMatchFunction((matchFunction)connection_comparator);
 
   time_t minDeath = 0;
@@ -457,6 +491,13 @@ void handleOidcdComm(struct ipcPipe pipes, int sock, const char* msg,
                                          : oidc_sprintf(INT_RESPONSE_ERROR, oidc_errno);
       SEC_FREE_KEY_VALUES();
       continue;
+    } else if (strequal(_request, INT_REQUEST_VALUE_CONFIRMMYTOKEN)) {
+      char* data = askpass_getMytokenConfirmation(_info);
+      send = data != NULL ? oidc_sprintf(RESPONSE_SUCCESS_INFO_OBJECT, data)
+                          : oidc_sprintf(INT_RESPONSE_ERROR, oidc_errno);
+      secFree(data);
+      SEC_FREE_KEY_VALUES();
+      continue;
     } else if (strequal(_request, INT_REQUEST_VALUE_QUERY_ACCDEFAULT)) {
       char* account = NULL;
       if (strValid(_issuer)) {  // default for this issuer
