@@ -1,15 +1,21 @@
 #include "gen_handler.h"
 
+#include <ctype.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
+#ifdef __MSYS__
+#include <sys/select.h>
+#endif
 
 #include "account/account.h"
 #include "account/issuer_helper.h"
 #include "defines/agent_values.h"
 #include "defines/ipc_values.h"
+#include "defines/mytoken_values.h"
 #include "defines/oidc_values.h"
 #include "defines/settings.h"
 #include "ipc/cryptCommunicator.h"
@@ -19,10 +25,8 @@
 #include "oidc-gen/gen_signal_handler.h"
 #include "oidc-gen/parse_ipc.h"
 #include "oidc-gen/promptAndSet/promptAndSet.h"
-#include "oidc-token/parse.h"
 #include "utils/accountUtils.h"
 #include "utils/crypt/crypt.h"
-#include "utils/crypt/cryptUtils.h"
 #include "utils/crypt/gpg/gpg.h"
 #include "utils/errorUtils.h"
 #include "utils/file_io/cryptFileUtils.h"
@@ -32,6 +36,7 @@
 #include "utils/listUtils.h"
 #include "utils/logger.h"
 #include "utils/oidc/device.h"
+#include "utils/oidc/oidcUtils.h"
 #include "utils/parseJson.h"
 #include "utils/password_entry.h"
 #include "utils/printer.h"
@@ -39,6 +44,9 @@
 #include "utils/promptUtils.h"
 #include "utils/string/stringUtils.h"
 #include "utils/uriUtils.h"
+#ifdef __MSYS__
+#include "utils/registryConnector.h"
+#endif
 
 #define IGNORE_ERROR 1
 
@@ -50,6 +58,7 @@ static const unsigned char remote = 0;
 
 char* _gen_response(struct oidc_account*    account,
                     const struct arguments* arguments) {
+  readConfigEndpoint(account, arguments);
   readDeviceAuthEndpoint(account, arguments);
   readAudience(account, arguments);
   cJSON* flow_json = listToJSONArray(arguments->flows);
@@ -70,6 +79,9 @@ char* _gen_response(struct oidc_account*    account,
       flow_json = jsonArrayAddStringValue(flow_json, FLOW_VALUE_PASSWORD);
     }
   }
+  if (strValid(account_getMytokenUrl(account))) {
+    flow_json = jsonArrayAddStringValue(flow_json, FLOW_VALUE_MT_OIDC);
+  }
   char* flow = jsonToString(flow_json);
   secFreeJson(flow_json);
   logger(DEBUG, "flows in handleGen are '%s'", flow);
@@ -294,7 +306,11 @@ void handleCodeExchange(const struct arguments* arguments) {
   char*  socket_path        = NULL;
   if (socket_path_base64 == NULL) {
     logger(NOTICE, "No socket_path encoded in state");
+#ifdef __MSYS__
+    socket_path = getRegistryValue(OIDC_SOCK_ENV_NAME);
+#else
     socket_path = oidc_strcopy(getenv(OIDC_SOCK_ENV_NAME));
+#endif
     if (socket_path == NULL) {
       printError("Socket path not encoded in url state and not available from "
                  "environment. Cannot connect to oidc-agent.\n");
@@ -457,6 +473,9 @@ struct oidc_account* manual_genNewAccount(struct oidc_account*    account,
   }
   if (account == NULL) {
     account = secAlloc(sizeof(struct oidc_account));
+    if (arguments->oauth) {
+      account_setOAuth2(account);
+    }
   }
   if (!arguments->only_at) {
     needName(account, arguments);
@@ -496,6 +515,63 @@ struct oidc_account* manual_genNewAccount(struct oidc_account*    account,
     }
   }
   readCertPath(account, arguments);
+  readConfigEndpoint(account, arguments);
+  if (MYTOKEN_USAGE_SET(arguments)) {
+    needMytokenIssuer(account, arguments);
+    char*   providers   = gen_handleMytokenProvidersLookup(account);
+    list_t* providers_l = JSONArrayStringToList(providers);
+    secFree(providers);
+    list_t* iss_l = list_new();
+    //    list_t* scopes_l = list_new();
+    //    scopes_l->free   = _secFree;
+    iss_l->free = _secFree;
+    list_node_t*     node;
+    list_iterator_t* it          = list_iterator_new(providers_l, LIST_HEAD);
+    unsigned char    foundArgIss = 0;
+    while ((node = list_iterator_next(it))) {
+      char* p   = node->val;
+      char* iss = getJSONValueFromString(p, OIDC_KEY_ISSUER);
+      if (arguments->issuer && compIssuerUrls(arguments->issuer, iss)) {
+        foundArgIss = 1;
+        break;
+      }
+      list_rpush(iss_l, list_node_new(iss));
+      //      list_rpush(scopes_l, list_node_new(getJSONValueFromString(
+      //                               p, OIDC_KEY_SCOPES_SUPPORTED)));
+    }
+    list_iterator_destroy(it);
+    secFreeList(providers_l);
+    if (arguments->issuer != NULL) {
+      secFreeList(iss_l);
+      if (foundArgIss) {
+        account_setIssuerUrl(account, oidc_strcopy(arguments->issuer));
+      } else {
+        char* e = oidc_sprintf("The specified issuer '%s' is not supported by "
+                               "this mytoken server.\n",
+                               arguments->issuer);
+        printError(e);
+        secFree(e);
+        exit(EXIT_FAILURE);
+      }
+    } else {
+      _suggestTheseIssuers(iss_l, account, 0);
+    }
+    //    const char* iss = account_getIssuerUrl(account);
+    //    for (size_t i = 0; i < iss_l->len; i++) {
+    //      if (compIssuerUrls(list_at(iss_l, i)->val, iss)) {
+    //        _askOrNeedScope(
+    //            JSONArrayStringToDelimitedString(list_at(scopes_l, i)->val, "
+    //            "), account, arguments, 0);
+    //        removeScope(account_getScope(account), OIDC_SCOPE_OFFLINE_ACCESS);
+    //        break;
+    //      }
+    //    }
+    //    secFreeList(iss_l);
+    //    secFreeList(scopes_l);
+    readMyProfile(account, arguments);
+    readRefreshToken(account, arguments);
+    return account;
+  }
   needIssuer(account, arguments);
   needClientId(account, arguments);
   askOrNeedClientSecret(account, arguments, arguments->usePublicClient);
@@ -525,6 +601,9 @@ struct oidc_account* registerClient(struct arguments* arguments) {
     exit(EXIT_FAILURE);
   }
   struct oidc_account* account = secAlloc(sizeof(struct oidc_account));
+  if (arguments->oauth) {
+    account_setOAuth2(account);
+  }
   needName(account, arguments);
   if (oidcFileDoesExist(account_getName(account))) {
     printError("An account with that shortname is already configured\n");
@@ -550,6 +629,7 @@ struct oidc_account* registerClient(struct arguments* arguments) {
   secFree(tmpData);
 
   readCertPath(account, arguments);
+  readConfigEndpoint(account, arguments);
   needIssuer(account, arguments);
   readDeviceAuthEndpoint(account, arguments);
   needScope(account, arguments);
@@ -663,12 +743,15 @@ struct oidc_account* registerClient(struct arguments* arguments) {
     secFreeJson(account_config_json);
     secFreeJson(client_config_json);
     char* new_scope_value = getJSONValue(merged_json, OIDC_KEY_SCOPE);
-    if (!strSubStringCase(new_scope_value, OIDC_SCOPE_OPENID) ||
+    if ((!strSubStringCase(new_scope_value, OIDC_SCOPE_OPENID) &&
+         account_getIsOAuth2(account)) ||
         !strSubStringCase(new_scope_value, OIDC_SCOPE_OFFLINE_ACCESS)) {
-      printError("Registered client does not have all the required scopes: %s "
-                 "%s\nPlease contact the provider to update the client to have "
+      printError("Registered client does not have all the required scopes: %s\n"
+                 "Please contact the provider to update the client to have "
                  "all the requested scope values.\n",
-                 OIDC_SCOPE_OPENID, OIDC_SCOPE_OFFLINE_ACCESS);
+                 account_getIsOAuth2(account) ? OIDC_SCOPE_OFFLINE_ACCESS
+                                              : OIDC_SCOPE_OPENID
+                     " " OIDC_SCOPE_OFFLINE_ACCESS);
       printIssuerHelp(account_getIssuerUrl(account));
       secFree(new_scope_value);
       secFreeJson(merged_json);
@@ -871,7 +954,7 @@ oidc_error_t gen_saveAccountConfig(const char* config, const char* shortname,
   if (arguments->noSave) {
     return OIDC_SUCCESS;
   }
-  char* tmpFile = oidc_strcat(CLIENT_TMP_PREFIX, shortname);
+  char* tmpFile = oidc_pathcat(CLIENT_TMP_PREFIX, shortname);
   char* tmpData = readFileFromAgent(tmpFile, IGNORE_ERROR);
   if (oidc_gen_state.doNotMergeTmpFile || tmpData == NULL) {
     secFree(tmpFile);
@@ -1017,9 +1100,11 @@ oidc_error_t gen_handlePublicClient(struct oidc_account* account,
   return OIDC_SUCCESS;
 }
 
-char* gen_handleScopeLookup(const char* issuer_url, const char* cert_path) {
-  char* res =
-      ipc_cryptCommunicate(remote, REQUEST_SCOPES, issuer_url, cert_path);
+char* gen_handleScopeLookup(const struct oidc_account* account) {
+  const char* iss = account_getIssuerUrl(account);
+  char*       res = ipc_cryptCommunicate(remote, REQUEST_SCOPES, iss,
+                                         account_getConfigEndpoint(account),
+                                         account_getCertPath(account));
 
   INIT_KEY_VALUE(IPC_KEY_STATUS, OIDC_KEY_ERROR, IPC_KEY_INFO);
   if (CALL_GETJSONVALUES(res) < 0) {
@@ -1032,8 +1117,8 @@ char* gen_handleScopeLookup(const char* issuer_url, const char* cert_path) {
   secFree(res);
   KEY_VALUE_VARS(status, error, scopes);
   if (!strequal(_status, STATUS_SUCCESS)) {
-    printError("Error while retrieving supported scopes for '%s': %s\n",
-               issuer_url, _error);
+    printError("Error while retrieving supported scopes for '%s': %s\n", iss,
+               _error);
     SEC_FREE_KEY_VALUES();
     exit(EXIT_FAILURE);
   }
@@ -1041,6 +1126,32 @@ char* gen_handleScopeLookup(const char* issuer_url, const char* cert_path) {
   return _scopes;
 }
 
+char* gen_handleMytokenProvidersLookup(const struct oidc_account* account) {
+  const char* iss = account_getMytokenUrl(account);
+  char*       res = ipc_cryptCommunicate(remote, REQUEST_MYTOKEN_PROVIDERS, iss,
+                                         account_getConfigEndpoint(account),
+                                         account_getCertPath(account));
+
+  INIT_KEY_VALUE(IPC_KEY_STATUS, OIDC_KEY_ERROR, IPC_KEY_INFO);
+  if (CALL_GETJSONVALUES(res) < 0) {
+    printError("Could not decode json: %s\n", res);
+    printError("This seems to be a bug. Please hand in a bug report.\n");
+    secFree(res);
+    SEC_FREE_KEY_VALUES();
+    exit(EXIT_FAILURE);
+  }
+  secFree(res);
+  KEY_VALUE_VARS(status, error, providers);
+  if (!strequal(_status, STATUS_SUCCESS)) {
+    printError("Error while retrieving supported providers for '%s': %s\n", iss,
+               _error);
+    SEC_FREE_KEY_VALUES();
+    exit(EXIT_FAILURE);
+  }
+  secFree(_status);
+  return _providers;
+}
+
 char* readFileFromAgent(const char* filename, int ignoreError) {
   char* res = ipc_cryptCommunicate(remote, REQUEST_FILEREAD, filename);
   if (res == NULL) {
@@ -1104,13 +1215,16 @@ void handleOnlyAT(struct arguments* arguments) {
   struct oidc_account* account = NULL;
   if (!arguments->manual) {  // On default try using a public client.
     account = secAlloc(sizeof(struct oidc_account));
+    readConfigEndpoint(account, arguments);
     needIssuer(account, arguments);
     updateAccountWithPublicClientInfo(account);
     arguments->usePublicClient = 1;
-  }
-  if (arguments->file) {
+  } else if (arguments->file) {
     account = getAccountFromMaybeEncryptedFile(arguments->file);
   }
+  if (arguments->oauth) {
+    account_setOAuth2(account);
+  }
   account = manual_genNewAccount(account, arguments, NULL);
 
   char* at = _gen_response(account, arguments);
