@@ -26,7 +26,8 @@ struct arguments {
   struct optional_arg expiration_env;
   struct optional_arg token_env;
 
-  unsigned char seccomp;
+  struct optional_arg mytoken;
+
   unsigned char printAll;
   unsigned char idtoken;
   unsigned char forceNewToken;
