@@ -13,13 +13,18 @@
 void common_handleListConfiguredAccountConfigs() {
   list_t* list = getAccountConfigFileList();
   if (list == NULL) {
-    oidc_perror();
-    exit(EXIT_FAILURE);
+    if (oidc_errno != OIDC_SUCCESS) {
+      oidc_perror();
+      exit(EXIT_FAILURE);
+    }
+    printStdoutIfTTY("No account configurations created yet.\n");
+    exit(EXIT_SUCCESS);
   }
   list_mergeSort(list, (int(*)(const void*, const void*))compareFilesByName);
   char* str = listToDelimitedString(list, "\n");
   secFreeList(list);
-  printStdout("The following account configurations are usable: \n%s\n", str);
+  printStdoutIfTTY("The following account configurations are usable: \n");
+  printStdout("%s\n", str);
   secFree(str);
 }
 
