@@ -1,6 +1,7 @@
 #ifndef FILE_UTILS_H
 #define FILE_UTILS_H
 
+#include "defines/msys.h"
 #include "utils/oidc_error.h"
 #include "wrapper/list.h"
 
@@ -19,6 +20,10 @@ int compareOidcFilesByDateAccessed(const char* filename1,
 char* generateClientConfigFileName(const char* issuer_url,
                                    const char* client_id);
 
+#ifndef MINGW
 oidc_error_t changeGroup(const char* path, const char* group_name);
+#endif
+
+char* fillEnvVarsInPath(const char* path_in);
 
 #endif  // FILE_UTILS_H
