@@ -4,6 +4,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 
+#include "defines/msys.h"
 #include "utils/oidc_error.h"
 #include "wrapper/list.h"
 
@@ -17,6 +18,7 @@ oidc_error_t writeFile(const char* filepath, const char* text);
 oidc_error_t appendFile(const char* path, const char* text);
 char*        readFile(const char* path);
 char*        readFILE(FILE* fp);
+oidc_error_t readBinaryFile(const char* path, char** buffer, size_t* size);
 char*        getLineFromFILE(FILE* fp);
 char*        getLineFromFile(const char* path);
 int          fileDoesExist(const char* path);
@@ -26,5 +28,10 @@ int          removeFile(const char* path);
 list_t*      getLinesFromFile(const char* path);
 list_t*      getLinesFromFileWithoutComments(const char* path);
 oidc_error_t mkpath(const char* p, mode_t mode);
+char*        getExistingLocation(list_t* possibleLocations);
+
+#ifdef MINGW
+int getline(char** lineptr, size_t* n, FILE* stream);
+#endif
 
 #endif  // FILE_IO_H
