@@ -6,7 +6,7 @@
 #define LIST_CREATE_COPY_VALUES 1
 #define LIST_CREATE_DONT_COPY_VALUES 0
 
-typedef int (*matchFunction)(const void*, const void*);
+typedef int (*matchFunction)(void*, void*);
 typedef void (*freeFunction)(void*);
 
 char*        delimitedStringToJSONArray(char* str, char delimiter);
@@ -26,5 +26,6 @@ void         secFreeList(list_t* l);
 list_t*      createList(int copyValues, char* s, ...);
 list_t*      list_addStringIfNotFound(list_t* l, char* v);
 int          listValid(const list_t* l);
+list_node_t* list_ats(list_t* l, size_t pos);
 
 #endif  // LIST_UTILS_H
